% function [p errorDist] = randomizationTest(A,B,amount,showgraph)
%
% This is a non-parametric test for the significance of a claim that the
% magnitude of the differences in errors between two methods of making
% predictions is greater than or equal to some amount.
%
% This function takes three or four parameters:
% - A: a vector of prediction errors achieved using method 1
% - B: a vector of prediction errors on the same instances, using method 2
% - amount: the claim is that Error(A) - Error(B) is at least 'amount'
%   (that is, method 2 is superior and by at least "amount").
%   As implemented here, RMSE is used as the measure of error, but this 
%   would be easy to change.
% - showGraph: boolean value indicating whether to show a graph of
%   claimed amount of error difference between methods 1 and 2 and the
%   corresponding confidence level at which this difference is significant.
%   Default value is true.
% What is returned is the probability that the true difference in
% prediction accuracy is less than 'amount'
%
% Here's an example of how to run the function by testing two random sets 
% of predictions:  randomizationTest(rand(2000,1),rand(2000,1),.1,true)
% To increase the accuracy of the test: (a) input more predictions; (b)
% carry out more trials.  Observe what happens when these changes are made
% using this example.
%
% Here's how the test works:
%
% randomization test: (1) take two methods A, B, that make predictions on a data
% series.  The goal is to determine whether a given difference in
% prediction accuracy Err(A) - Err(B) is statistically significant. (1) take the
% predictions of the two methods and randomly shuffle the labels A and B.
% (2) recompute Err(A) - Err(B) (3) repeat 1 and 2 10,000 times, and build
% up an empirical distribution of Err(A) - Err(B).  The fraction of these
% samples in the empirical distribution with a smaller difference in
% prediction difference is the confidence level at which the hypothesis
% "the difference between A and B is at least X" would be accepted.

function [p errorDist] = randomizationTest(A,B,amount,showgraph)

	error(nargchk(3,4,nargin));
	if (nargin == 3)
		showgraph = true;
	end

	trials = 10000;

	% allocate memory
	newA = A;
	newB = B;
	errorDist = zeros(trials,1);

	% run 'trials' independent trials
	for i=1:trials
		for j=1:size(A)
			% randomly assign elements of A and B to newA and newB
			if (rand() > 0.5)
				newA(j) = A(j);
				newB(j) = B(j);
			else
				newA(j) = B(j);
				newB(j) = A(j);
			end
		end

		% compute the difference in prediction accuracy between newA and newB 
		newB = newB .^ 2;
		newA = newA .^ 2;
		errorDist(i,1) = sqrt(mean(newA)) - sqrt(mean(newB));
	end

	% empirical distribution of errorDist
	errorDist = sort(errorDist);
	
	% graph it:
	if (showgraph)
		probs = [1:trials] ./ trials;
		plot(errorDist, probs)
		title('Randomization test');
		xlabel ('Observed difference in error between method 1 and method 2');
		ylabel ('Probability that the true difference is no larger than this');
	end
	
	p=1;
	for i=trials:-1:1
		if (errorDist(i) < amount)
            %I think this is a bug, should be i/trials: en
			p = (trials-i)/trials;           
			break;
		end
	end
end
		
	