function dataframes = readDF(trainfilename, testfilename,  validfilename, numalgs, MinTime, cappedIncluded, isPortfolioData, removeCapped)
% dataframes = readDF(trainfilename, testfilename,  validfilename, numalgs, MinTime, cappedIncluded, isPortfolioData, removeCapped)
%
% readDF returns an array of dataframes, one for each algorithm
% Parameters are filenames of train and test and validation datasets, and number of
% algorithm columns in each filename
% MinTime - increase all small times to mintime.
% 
% if cappedIncluded is true, then columns following algorithm columns are
% assumed to be capped flags. These are removed for test/valid sets
%
% If portfolioData is true, then last column before features is assumed to
% be feature computation times. In that case, only test instances for which
% _all_ algorithms timed out are removed.

error( nargchk(8,8,nargin));


traindata = csvread(trainfilename, 1, 0);
testdata = csvread(testfilename,1,0);
validdata = csvread(validfilename,1,0);


algIdx = 1:numalgs;
nextIdx=numalgs+1;

if cappedIncluded 
    cappedIdx=nextIdx:nextIdx+numalgs-1;
    nextIdx=nextIdx+numalgs;

    if removeCapped        
        
        if isPortfolioData
            
            %figure out least captime. Use common cap for all algorithms
            commonCap = Inf;    
            for t=1:numalgs
                if(max(testdata(:,numalgs+t))>0)
                    commonCap = min(commonCap, min(testdata(testdata(:,numalgs+t)==1,t)));
                end  
                
                if  (max(validdata(:,numalgs+t))>0)
                    commonCap = min(commonCap, min(validdata(validdata(:,numalgs+t)==1,t)));
                end
                
            end
            
            disp(['Common captime for portfolio: ' sprintf('%f', commonCap)]);
            goodTestRows = (prod( single(testdata(:,algIdx) >= commonCap), 2) ==0 );
            goodValidRows = (prod( single(validdata(:,algIdx) >= commonCap), 2) == 0 );
        else
            goodTestRows = (prod(single(testdata(:,cappedIdx)),2) == 0);
            goodValidRows = (prod(single(validdata(:,cappedIdx)),2) == 0);
        end
        
        testdata = testdata(goodTestRows,:);
        validdata = validdata(goodValidRows,:);
    end

    traincaps = traindata(:,cappedIdx);
    testcaps = testdata(:,cappedIdx);
    validcaps = validdata(:,cappedIdx);    
end

if isPortfolioData
    featTimeIdx=nextIdx;
    nextIdx=nextIdx+1;
    
    trainFeatTimes = traindata(:,featTimeIdx);
    testFeatTimes = testdata(:,featTimeIdx);
    validFeatTimes = validdata(:,featTimeIdx);
end


traintimes = traindata(:,algIdx);
testtimes = testdata(:,algIdx);
validtimes = validdata(:,algIdx);


trainfeatures = traindata(:,nextIdx:end);
testfeatures = testdata(:,nextIdx:end);
validfeatures = validdata(:,nextIdx:end);


allnames = textread(trainfilename, '%s', 1, 'whitespace', '\n');
allnames = strread(allnames{1},'%s','whitespace',',');
allnames = deblank(allnames);

featurenames = allnames(nextIdx:end);
featurenames = featurenames';

for i =1:numalgs
    % infer the captime; set the mintime.
    dataframes(i).captime = max(max(max(traintimes(:,i))),max(max(testtimes(:,i))));
    dataframes(i).captime = max(dataframes(i).captime, max(max(validtimes(:,i))))+30;
    dataframes(i).mintime = MinTime;
    
    dataframes(i).traindata.origResponse = traintimes(:,i);
    dataframes(i).testdata.origResponse = testtimes(:,i);
    dataframes(i).validdata.origResponse = validtimes(:,i);
    
    % round up small runtimes
    dataframes(i).traindata.response = max(dataframes(i).mintime, traintimes(:,i));
    dataframes(i).testdata.response =  max(dataframes(i).mintime, testtimes(:,i));
    dataframes(i).validdata.response = max(dataframes(i).mintime, validtimes(:,i));
        
    dataframes(i).traindata.features = trainfeatures;
    dataframes(i).testdata.features = testfeatures;
    dataframes(i).validdata.features = validfeatures;
    
    dataframes(i).featurenames = featurenames;
    dataframes(i).algname = allnames{i};

    if cappedIncluded
        dataframes(i).traindata.capped = traincaps(i,:);    
        dataframes(i).testdata.capped = testcaps(i,:);
        dataframes(i).validdata.capped = validcaps(i,:);
    end
    
    if isPortfolioData
        dataframes(i).traindata.featureTimes = trainFeatTimes;    
        dataframes(i).testdata.featureTimes = testFeatTimes;
        dataframes(i).validdata.featureTimes = validFeatTimes;
    end
    
    
    
    %no transform to start with
    dataframes(i).backtransform = @IDTransform;
    
end


