function subs = sequentialReplacement(df, maxSize)
%function subs = subsetSelection(df, maxSize, type)
%
%Do subset selection on df, using the forward selection heuristic, looking
%at subsets of increasing size. 
%Returns a struct array of size 1..maxSize, containing selected features
%vector (colidx), rmse and mae of the model (on training data)

error(nargchk(2,2,nargin));

display (sprintf('Sequential replacement started: %d features total',size(df.traindata.features,2)));
X = ones(size(df.traindata.features,1),1);
global delta;
A = X'*X+delta;
Ainv = inv(A);
XtY = X' * df.traindata.response;
compute_training_RMSE = false;
max_sequential_replacements = 5;

%test = feval(df.backtransform, df.validdata.response, df);
test = df.validdata.response;

features = [];
num_train_instances = size(df.traindata.features,1);
num_test_instances = size(df.validdata.features,1);
num_features = size(df.traindata.features,2);
maxIteration = min(maxSize,num_features);
Xtest = ones(num_test_instances,1);

% for every set size
for t=1:maxIteration
	
	minRMSE = inf;
	best_subset = 0;

	if (mod(t,1)==0)
		display (sprintf('  sequential replacement: subset size %d of %d', [t maxIteration]));
	end

	% ================================
	% PHASE 1: ADDING NEW FEATURES
	% ================================
	
	% for each feature not in the set
	for tt=1:size(df.traindata.features,2) %- size(features,2)
		if (~isempty(find(features==tt)))
			continue;
		end
		
		features(t)=tt;
	
		% construct the model with this feature added
		model = modelAddFeature(X, df.traindata.features(:,tt), df.traindata.response, A, Ainv, delta, XtY);

		% record the RMSE and MAE on validation data for this feature
		Xtest(:,t+1) = df.validdata.features(:,tt);
		preds = Xtest*model;
		
		result(tt).MAE = mean(abs(preds-test));
		result(tt).RMSE = sqrt(mean( (preds-test).^2 ));

		result(tt).features = features;
		result(tt).method = 'forwardSelection';
		minRMSE = min(minRMSE, result(tt).RMSE);
		if (minRMSE == result(tt).RMSE)
			best_subset = tt;
		end
	end

	% pick the feature with the best RMSE
	subs(t) = result(best_subset);
	clear result;
	features = subs(t).features;
	
	% update the inverse; global variables
	[model A Ainv XtY] = modelAddFeature(X, df.traindata.features(:,best_subset), df.traindata.response, A, Ainv, delta, XtY);
	X = [X df.traindata.features(:,best_subset)];
	Xtest(:,t+1) = df.validdata.features(:,best_subset);

	% ================================
	% PHASE 2: SWAPPING FEATURES
	% ================================
	increment = 0;
	while t > 1 && increment < max_sequential_replacements

		% the RMSE to beat is the minimum from the previous forward selection round (or the previous swap iteration) because that's our current model
		bestRMSEbefore = minRMSE; 
		index = 0;
		
		% pick the feature to remove
		for tt=1:size(df.traindata.features,2) %- size(features,2)
			if (isempty(find(features==tt)))
				continue;
			else
				k = find(features==tt);
			end

			% pick another feature to replace it with
			for ttt=1:size(df.traindata.features,2) %- size(features,2)
				if (~isempty(find(features==ttt)))
					continue;
				end

				% update the index used for the best found so far vector (an alternative to having to keep track of 2d indexing)
				index = index + 1;
				
				% remove feature k and add feature ttt
				model = modelSwapFeature(X, k, df.traindata.features(:,ttt), df.traindata.response, A, Ainv, delta, XtY);

				% compute error, keep track of best, etc.
				preds = [Xtest(:,1:end~=k) df.validdata.features(:,tt)] * model;

				result(index).MAE = -1;  % don't store something meaningful here
				result(index).RMSE = sqrt(mean( (preds-test).^2 ));

				temp = features(1:end~=find(features==tt));
				temp(end+1) = ttt;
				result(index).features = temp;
				result(index).method = 'sequentialReplacement';
				r(index).removed_feature = k; % put it into a different structure so that results can be merged with forward, backward selection
				r(index).added_feature = ttt;
				minRMSE = min(minRMSE, result(index).RMSE);
				%display (sprintf('current RMSE %f minRMSE %f bestRMSEbefore %f',result(index).RMSE,minRMSE,bestRMSEbefore))
				if (minRMSE == result(index).RMSE)
					best_subset = index;
				end
			end
		end
		
		% if nothing was good, finish this round of sequential replacement
		if (minRMSE == bestRMSEbefore)
			increment = max_sequential_replacements;
		
		% otherwise, implement the best swap, and update the loop counter by one
		else
			display (sprintf ('  -- swap was worthwhile: RMSE = %f; original RMSE = %f',minRMSE,bestRMSEbefore));
			[model A Ainv XtY] = modelSwapFeature(X, r(best_subset).removed_feature, df.traindata.features(:,r(best_subset).added_feature), df.traindata.response, A, Ainv, delta, XtY);
			X = [X(:,1:end~=r(best_subset).removed_feature) df.traindata.features(:,r(best_subset).added_feature)];
			Xtest = [Xtest(:,1:end~=r(best_subset).removed_feature) df.validdata.features(:,r(best_subset).added_feature)];
			preds = Xtest * model; 
			result(best_subset).MAE = mean(abs(preds-test));  % now compute it, only for the one we're keeping

			subs(t) = result(best_subset);
			features = subs(t).features;

			clear result;
			clear r;
			increment = increment + 1;
		end
	end
end

display 'Sequential replacement finished'